﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;

//using HM_API;  // the namespace exposes the library interface 

/* ADC Continuous Callback acquisition Sample
 * This sample acquires using a callback function (myCallBack) which stores the voltages received into a global array ... but overwrites most of the data
 * saving only the last scan worth.
 * The sample displays data in a timer event handler at 10Hz, simply showing the most recent scan of data as acquired by the Callback.
 * 
 * In a real-world application the callback wouldn't overwrite the Volts[] values, it might log or FFT or whatnot instead.
 * 
 */


namespace Sample1
{
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void ADCallback(IntPtr pBuf, UInt32 BufSize, UInt32 Flags, UInt32 Context);

    public partial class Form1 : Form
    {
        public static ADCallback myCallBack;

        public UInt32 DeviceIndex;        
        public CheckBox[] checkStateArray = new CheckBox[16];
        public ComboBox[] DACRanges = new ComboBox[4];
        public Label[] DACLabels = new Label[4];
        public double[] Volts = new double[8];
        public int counter;
        public TrackBar[] DACSliders = new TrackBar[4];

        public Form1()
        {
            InitializeComponent();
            myCallBack = new ADCallback(ADCallbackReport);
        }
        
        private void Form1_Load(object sender, EventArgs e)
        {
            // Array of the LED style checkboxes:
            checkStateArray[0] = checkBoxA0_1;
            checkStateArray[1] = checkBoxA0_2;
            checkStateArray[2] = checkBoxA0_3;
            checkStateArray[3] = checkBoxA0_4;
            checkStateArray[4] = checkBoxA0_5;
            checkStateArray[5] = checkBoxA0_6;
            checkStateArray[6] = checkBoxA0_7;
            checkStateArray[7] = checkBoxA0_8;
            checkStateArray[8] = checkBoxB0_9;
            checkStateArray[9] = checkBoxB0_10;
            checkStateArray[10] = checkBoxB0_11;
            checkStateArray[11] = checkBoxB0_12;
            checkStateArray[12] = checkBoxB0_13;
            checkStateArray[13] = checkBoxB0_14;
            checkStateArray[14] = checkBoxB0_15;
            checkStateArray[15] = checkBoxB0_16;
            
            DACRanges[0] = DACRange0;
            DACRanges[1] = DACRange1;
            DACRanges[2] = DACRange2;
            DACRanges[3] = DACRange3;

            DACLabels[0] = DACLabel0;
            DACLabels[1] = DACLabel1;
            DACLabels[2] = DACLabel2;
            DACLabels[3] = DACLabel3;

            DACSliders[0] = DACCounts0;
            DACSliders[1] = DACCounts1;
            DACSliders[2] = DACCounts2;
            DACSliders[3] = DACCounts3;

            DeviceIndex = 0; // Initialize default Device Only
            RangeComboBox0.SelectedIndex = 7;
            RangeComboBox1.SelectedIndex = 7;
            RangeComboBox2.SelectedIndex = 7;
            RangeComboBox3.SelectedIndex = 7;
            RangeComboBox4.SelectedIndex = 7;
            RangeComboBox5.SelectedIndex = 7;
            RangeComboBox6.SelectedIndex = 7;
            RangeComboBox7.SelectedIndex = 7;

            DACRange0.SelectedIndex = 3;
            DACRange1.SelectedIndex = 3;
            DACRange2.SelectedIndex = 3;
            DACRange3.SelectedIndex = 3;

            // DeviceIndex = HM_API.Open(0);
            if (DeviceIndex < 0) /* handle Open error */;
        }

        private void ExecuteButton_Click(object sender, EventArgs e)
        {
            ExecuteButton.Enabled = false;
            /*
                        // each RangeCombo holds the gaincode in the right order so we can pass the SelectedIndex directly as the gain code
                        HM_API.ADC_SetRanges(DeviceIndex, RangeComboBox0.SelectedIndex,
                            RangeComboBox1.SelectedIndex, 
                            RangeComboBox2.SelectedIndex, 
                            RangeComboBox3.SelectedIndex, 
                            RangeComboBox4.SelectedIndex, 
                            RangeComboBox5.SelectedIndex, 
                            RangeComboBox6.SelectedIndex, 
                            RangeComboBox7.SelectedIndex
                        );
                        HM_API.ADC_SetRate(DeviceIndex, 60.0); // Hertz, scans
                        HM_API.ADC_StartCallback(DeviceIndex, 0 *//* context *//*, myCallBack);
            */
            StopButton.Enabled = true;
        }

        public void ADCallbackReport(IntPtr pBuf, UInt32 BufSize, UInt32 Flags, UInt32 Context)
        {
            if (((Flags & 1) != 0)) // This flag tells us we've got the last continuous callback packet:
            {
                // in case we care               
            }

            double[] Buf = new double[BufSize/sizeof(double)]; // array for marshalling
            Marshal.Copy(pBuf, Buf, 0, Buf.Length);
            for (int i = 0; i < Buf.Length; i++)
            {
                if (cbDifferential.Checked)
                    Volts[(i * 2) % ADCLastChannel.SelectedIndex] = Buf[i];
                else
                    Volts[i % ADCLastChannel.SelectedIndex] = Buf[i]
                ;
            }
            counter++;
        }

        private void UpdateDIOState()
        {
            byte[] Data = new byte[2];

            // HM_API.DIO_Read(DeviceIndex, out Data);

            // Test each bit and set check state:
            for (int i = 0; i < 16; i++)
            {
                if ((Data[i / 8] & (1 << (i % 8))) != 0) // looks complicated but efficient
                    checkStateArray[i].Checked = true; // High Level
                else
                    checkStateArray[i].Checked = false; // Low Level
            }
        }

        private void DisplayTimer_Tick(object sender, EventArgs e)
        {
            listBoxVoltage.Items.Clear(); 
            for (int I = 0; I < 8; ++I)
            {
                string strChVolt = String.Format("Ch {0,2:D2}: {1,8:F4}", I, Volts[I]);
                listBoxVoltage.Items.Add(strChVolt);
            }
            listBoxVoltage.Items.Add("Total callbacks: "+counter.ToString());

            if (PollDIOCheckBox.Checked) 
                UpdateDIOState();
        }

        private void StopButton_Click(object sender, EventArgs e)
        {
            StopButton.Enabled = false;

            // HM_API.ADC_StopCallback(DeviceIndex, 0 /* context */);

            ExecuteButton.Enabled = true;
        }

        private void cbDifferential_CheckedChanged(object sender, EventArgs e)
        {
            switch (cbDifferential.Checked)
            {
                case true:
                    RangeComboBox1.Visible = false; label1.Visible = false; 
                    RangeComboBox3.Visible = false; label3.Visible = false; 
                    RangeComboBox5.Visible = false; label5.Visible = false; 
                    RangeComboBox7.Visible = false; label7.Visible = false; 
                    ADCLastChannel.Items.Clear();
                    ADCLastChannel.Items.AddRange(new string[] { "0", "2", "4", "6" });
                    ADCLastChannel.SelectedIndex = 3;
                    break;
                    
                case false:
                    RangeComboBox1.Visible = true; label1.Visible = true; 
                    RangeComboBox3.Visible = true; label3.Visible = true; 
                    RangeComboBox5.Visible = true; label5.Visible = true; 
                    RangeComboBox7.Visible = true; label7.Visible = true;
                    ADCLastChannel.Items.Clear();
                    ADCLastChannel.Items.AddRange(new string[] { "0", "1", "2", "3", "4", "5", "6", "7" });
                    ADCLastChannel.SelectedIndex = 7;
                    break;

                default:
                    break;
            }
        }

        private void btnToggleDIODirection_Click(object sender, EventArgs e)
        {
            // Button Clicks to toggle the input output mode and display states:
            bool DIOByte0isOutput, DIOByte1isOutput;
            DIOByte0isOutput = btnToggle0.Text == "Toggle to Output Mode";
            DIOByte1isOutput = btnToggle1.Text == "Toggle to Output Mode";

            Button btnSender = sender as Button;

            if (btnSender.Text == "Toggle to Input Mode")
                btnSender.Text = "Toggle to Output Mode";
            else
                btnSender.Text = "Toggle to Input Mode";

            // HM_API.DIO_Configure(DeviceIndex, DIOByte0isOutput, DIOByte1isOutput);
        }

        private void DACValueChanged(object sender, EventArgs e)
        {
            int DAC = Convert.ToInt32((sender as TrackBar).Tag);
            double Volts = (sender as TrackBar).Value / 65536.0;
            switch (DACRanges[DAC].SelectedIndex)
            {
                case 0: Volts *= 5.0; break; // unipolar 5
                case 1: Volts *= 10.0; break; // unipolar 10
                case 2: Volts *= 10.0; Volts -= 5.0; break; // ±5
                case 3: Volts *= 20.0; Volts -= 10.0; break; // ±10
                case 4: Volts *= 5.0; Volts -= 2.5; break; // ±2.5
            }
            DACLabels[DAC].Text = $"{Volts:F4}V";
            
            // HM_API.DAC_Update(DeviceIndex, DAC, Volts);
        }

        private void DACRange3_SelectedIndexChanged(object sender, EventArgs e)
        {
            int DAC = Convert.ToInt32((sender as ComboBox).Tag);
            // HM_API.DAC_Configure(DeviceIndex, DACRange0.SelectedIndex, DACRange1.SelectedIndex, DACRange2.SelectedIndex, DACRange3.SelectedIndex);
            if ((sender as ComboBox).SelectedIndex > 1)
                DACSliders[DAC].Value = 0x8000;
            else
                DACSliders[DAC].Value = 0x0000;
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            // HM_API.close(DeviceIndex);
        }
    }
}
